local L = AceLibrary("AceLocale-2.2"):new("sRaidFrames")
local sRaidFrames = sRaidFrames

function sRaidFrames:RegisterGroupSetup(name)
	self.opt.GroupSetups[name] = {}
	self:chatUpdateFilterMenu()
end

function sRaidFrames:GetGroupSetup(name)
	return self.opt.GroupSetups[name]
end

function sRaidFrames:RenameGroupSetup(old, new)
	self.opt.GroupSetups[new] = self.opt.GroupSetups[old]
	self.opt.GroupSetups[old] = nil
	if self.opt.GroupSetup == old then
		self.opt.GroupSetup = new
	end
	return self.opt.GroupSetups[new]
end

function sRaidFrames:DefaultGroupSetups()
	if not self:GetGroupSetup(L["By class"]) then
		self:RegisterGroupSetup(L["By class"])
		self:RegisterGroupFilter(L["By class"], 1, L["Warriors"], {["groupFilter"] = "WARRIOR"})
		self:RegisterGroupFilter(L["By class"], 2, L["Mages"], {["groupFilter"] = "MAGE"})
		self:RegisterGroupFilter(L["By class"], 3, L["Paladins"], {["groupFilter"] = "PALADIN"})
		self:RegisterGroupFilter(L["By class"], 4, L["Druids"], {["groupFilter"] = "DRUID"})
		self:RegisterGroupFilter(L["By class"], 5, L["Hunters"], {["groupFilter"] = "HUNTER"})
		self:RegisterGroupFilter(L["By class"], 6, L["Rogues"], {["groupFilter"] = "ROGUE"})
		self:RegisterGroupFilter(L["By class"], 7, L["Warlocks"], {["groupFilter"] = "WARLOCK"})
		self:RegisterGroupFilter(L["By class"], 8, L["Priests"], {["groupFilter"] = "PRIEST"})
		self:RegisterGroupFilter(L["By class"], 9, L["Shamans"], {["groupFilter"] = "SHAMAN"})
	end

	if not self:GetGroupSetup(L["By group"]) then
		self:RegisterGroupSetup(L["By group"])
		self:RegisterGroupFilter(L["By group"], 1, L["Group 1"], {["groupFilter"] = "1"})
		self:RegisterGroupFilter(L["By group"], 2, L["Group 2"], {["groupFilter"] = "2"})
		self:RegisterGroupFilter(L["By group"], 3, L["Group 3"], {["groupFilter"] = "3"})
		self:RegisterGroupFilter(L["By group"], 4, L["Group 4"], {["groupFilter"] = "4"})
		self:RegisterGroupFilter(L["By group"], 5, L["Group 5"], {["groupFilter"] = "5"})
		self:RegisterGroupFilter(L["By group"], 6, L["Group 6"], {["groupFilter"] = "6"})
		self:RegisterGroupFilter(L["By group"], 7, L["Group 7"], {["groupFilter"] = "7"})
		self:RegisterGroupFilter(L["By group"], 8, L["Group 8"], {["groupFilter"] = "8"})
	end

	if not self:GetGroupSetup(L["By role"]) then
		self:RegisterGroupSetup(L["By role"])
		self:RegisterGroupFilter(L["By role"], 1, L["Tanks"], {["groupFilter"] = "WARRIOR"})
		self:RegisterGroupFilter(L["By role"], 2, L["Melee DPS"], {["groupFilter"] = "ROGUE,WARRIOR"})
		self:RegisterGroupFilter(L["By role"], 3, L["Range DPS"], {["groupFilter"] = "MAGE,WARLOCK,HUNTER"})
		self:RegisterGroupFilter(L["By role"], 4, L["Healers"], {["groupFilter"] = "PALADIN,SHAMAN,PRIEST,DRUID"})
	end

	self:chatUpdateFilterMenu()
end

function sRaidFrames:DeleteGroupSetup(name)
	self.opt.GroupSetups[name] = nil
end

function sRaidFrames:GetGroupSetupFilter(name, group)
	return self.opt.GroupSetups[name][group]
end

function sRaidFrames:GetCurrentGroupSetup()
	return self:GetGroupSetup(self.opt.GroupSetup)
end

function sRaidFrames:RegisterGroupFilter(name, group, caption, attributes)
	if not attributes["sortMethod"] then
		attributes["sortMethod"] = "NAME"
	end
	if not attributes["sortDir"] then
		attributes["sortDir"] = "DESC"
	end

	self.opt.GroupSetups[name][group] = {["caption"] = caption, ["attributes"] = attributes}
	self:chatUpdateFilterMenu()
end

function sRaidFrames:DeleteGroupFilter(name, group)
	tremove(self.opt.GroupSetups[name], group)
	self:chatUpdateFilterMenu()
end

function sRaidFrames:SetCurrentGroupSetup(name)
	self:ScheduleLeaveCombatAction(function() self.opt.GroupSetup = name end)
	self:ScheduleLeaveCombatAction(self.CreateFrames, self)
	self:ScheduleLeaveCombatAction(self.SetGroupFilters, self)
	self:ScheduleLeaveCombatAction(self.Print, self, string.format(L["Current filter set is now '%s'"],name))
	self:chatUpdateFilterMenu()
end

function sRaidFrames:GetAllGroupFilters()
	return self.opt.GroupSetups
end

function sRaidFrames:chatUpdateFilterMenu()
	local i = 1
	for name, _ in pairs(self:GetAllGroupFilters()) do
		self.options.args.sets.args["set".. i] = {}
		self.options.args.sets.args["set".. i].type = 'group'
		self.options.args.sets.args["set".. i].name = L["Set: "] .. name .. ((name == self.opt.GroupSetup) and L[" |cff00ff00[Active]|r "] or "")
		self.options.args.sets.args["set".. i].desc = name
		self.options.args.sets.args["set".. i].args = {}

		if name ~= self.opt.GroupSetup then
			self.options.args.sets.args["set".. i].args["use"] = {}
			self.options.args.sets.args["set".. i].args["use"].type = 'execute'
			self.options.args.sets.args["set".. i].args["use"].name = L["|cff00ff00Activate this set!"]
			self.options.args.sets.args["set".. i].args["use"].desc = name
			self.options.args.sets.args["set".. i].args["use"].func = function() self:SetCurrentGroupSetup(name) self:chatUpdateFilterMenu() end
			self.options.args.sets.args["set".. i].args["use"].order = 1

			self.options.args.sets.args["set".. i].args["delete"] = {}
			self.options.args.sets.args["set".. i].args["delete"].type = 'execute'
			self.options.args.sets.args["set".. i].args["delete"].name = L["|cffff0000Delete this set"]
			self.options.args.sets.args["set".. i].args["delete"].desc = name
			self.options.args.sets.args["set".. i].args["delete"].func = function() self:DeleteGroupSetup(name) self:chatUpdateFilterMenu() end
			self.options.args.sets.args["set".. i].args["delete"].order = 300
		end

		self.options.args.sets.args["set".. i].args["rename"] = {}
		self.options.args.sets.args["set".. i].args["rename"].type = 'text'
		self.options.args.sets.args["set".. i].args["rename"].name = L["Rename this set"]
		self.options.args.sets.args["set".. i].args["rename"].desc = name
		self.options.args.sets.args["set".. i].args["rename"].get = function() return name end
		self.options.args.sets.args["set".. i].args["rename"].set = function(value)
																													self:RenameGroupSetup(name, value)
																													self:chatUpdateFilterMenu()
																													end
		self.options.args.sets.args["set".. i].args["rename"].usage = L["<Name>"]
		self.options.args.sets.args["set".. i].args["rename"].order = 2

		self.options.args.sets.args["set".. i].args["spacer1"] = {}
		self.options.args.sets.args["set".. i].args["spacer1"].type = 'header'
		self.options.args.sets.args["set".. i].args["spacer1"].order = 3

		self.options.args.sets.args["set".. i].args["spacer2"] = {}
		self.options.args.sets.args["set".. i].args["spacer2"].type = 'header'
		self.options.args.sets.args["set".. i].args["spacer2"].order = 200

		for id, data in pairs(self:GetGroupSetup(name)) do
			self.options.args.sets.args["set".. i].args["frame".. id] = {}
			self.options.args.sets.args["set".. i].args["frame".. id].type = 'group'
			self.options.args.sets.args["set".. i].args["frame".. id].name = L["Frame #"].. id .. " (".. data.caption ..")"
			self.options.args.sets.args["set".. i].args["frame".. id].desc = L["Frame "].. id
			self.options.args.sets.args["set".. i].args["frame".. id].args = {}

			self.options.args.sets.args["set".. i].args["frame".. id].args["caption"] = {}
			self.options.args.sets.args["set".. i].args["frame".. id].args["caption"].type = 'text'
			self.options.args.sets.args["set".. i].args["frame".. id].args["caption"].name = L["Caption"]
			self.options.args.sets.args["set".. i].args["frame".. id].args["caption"].desc = L["Caption"]
			self.options.args.sets.args["set".. i].args["frame".. id].args["caption"].usage = L["<Caption>"]
			self.options.args.sets.args["set".. i].args["frame".. id].args["caption"].get = function() return data.caption end
			self.options.args.sets.args["set".. i].args["frame".. id].args["caption"].set = function(value)
																																												self:RegisterGroupFilter(name, id, value, data.attributes)
																																												self:chatUpdateFilterMenu()
																																												self:SetGroupFilters()
																																											end


			self.options.args.sets.args["set".. i].args["frame".. id].args["remove"] = {}
			self.options.args.sets.args["set".. i].args["frame".. id].args["remove"].type = 'execute'
			self.options.args.sets.args["set".. i].args["frame".. id].args["remove"].name = L["|cffff0000Remove frame|r"]
			self.options.args.sets.args["set".. i].args["frame".. id].args["remove"].desc = L["Remove this frame from this filter"]
			self.options.args.sets.args["set".. i].args["frame".. id].args["remove"].order = 300
			self.options.args.sets.args["set".. i].args["frame".. id].args["remove"].func = function()
																																												self:DeleteGroupFilter(name, id)
																																												self:chatUpdateFilterMenu()
																																												self:SetGroupFilters()
																																											end

			self.options.args.sets.args["set".. i].args["frame".. id].args["sortmethod"] = {}
			self.options.args.sets.args["set".. i].args["frame".. id].args["sortmethod"].type = 'text'
			self.options.args.sets.args["set".. i].args["frame".. id].args["sortmethod"].name = L["Sort method"]
			self.options.args.sets.args["set".. i].args["frame".. id].args["sortmethod"].desc = L["Select which method to use for sorting"]
			self.options.args.sets.args["set".. i].args["frame".. id].args["sortmethod"].get = function() return data.attributes.sortMethod end
			self.options.args.sets.args["set".. i].args["frame".. id].args["sortmethod"].set = function(value)
																																												data.attributes.sortMethod = value
																																												self:RegisterGroupFilter(name, id, data.caption, data.attributes)
																																												self:chatUpdateFilterMenu()
																																												self:SetGroupFilters()
																																											end
			self.options.args.sets.args["set".. i].args["frame".. id].args["sortmethod"].validate = {["NAME"] = L["By player name"], ["INDEX"] = L["By position in group"]}


			self.options.args.sets.args["set".. i].args["frame".. id].args["sortdirection"] = {}
			self.options.args.sets.args["set".. i].args["frame".. id].args["sortdirection"].type = 'text'
			self.options.args.sets.args["set".. i].args["frame".. id].args["sortdirection"].name = L["Sort direction"]
			self.options.args.sets.args["set".. i].args["frame".. id].args["sortdirection"].desc = L["Select in which direction to sort players"]
			self.options.args.sets.args["set".. i].args["frame".. id].args["sortdirection"].validate = {["DESC"] = L["Descending"], ["ASC"] = L["Ascending"]}
			self.options.args.sets.args["set".. i].args["frame".. id].args["sortdirection"].get = function() return data.attributes.sortDir end
			self.options.args.sets.args["set".. i].args["frame".. id].args["sortdirection"].set = function(value)
																																												data.attributes.sortDir = value
																																												self:RegisterGroupFilter(name, id, data.caption, data.attributes)
																																												self:chatUpdateFilterMenu()
																																												self:SetGroupFilters()
																																											end

			self.options.args.sets.args["set".. i].args["frame".. id].args["strictfilter"] = {}
			self.options.args.sets.args["set".. i].args["frame".. id].args["strictfilter"].type = 'toggle'
			self.options.args.sets.args["set".. i].args["frame".. id].args["strictfilter"].name = L["Strict filtering"]
			self.options.args.sets.args["set".. i].args["frame".. id].args["strictfilter"].desc = L["If set, characters must match both a group and a class from the list"]
			self.options.args.sets.args["set".. i].args["frame".. id].args["strictfilter"].get = function() return data.attributes.strictFiltering or false end
			self.options.args.sets.args["set".. i].args["frame".. id].args["strictfilter"].set = function(value)
																																												data.attributes.strictFiltering = value
																																												self:RegisterGroupFilter(name, id, data.caption, data.attributes)
																																												self:chatUpdateFilterMenu()
																																												self:SetGroupFilters()
																																											end

			self.options.args.sets.args["set".. i].args["frame".. id].args["classes"] = {}
			self.options.args.sets.args["set".. i].args["frame".. id].args["classes"].type = 'group'
			self.options.args.sets.args["set".. i].args["frame".. id].args["classes"].name = L["Classes"]
			self.options.args.sets.args["set".. i].args["frame".. id].args["classes"].desc = L["Classes contained in this frame"]
			self.options.args.sets.args["set".. i].args["frame".. id].args["classes"].args = {}

			for eclass, class in pairs({["WARRIOR"] = L["Warriors"], ["PALADIN"] = L["Paladins"], ["SHAMAN"] = L["Shamans"], ["PRIEST"] = L["Priests"], ["ROGUE"] = L["Rogues"], ["HUNTER"] = L["Hunters"], ["MAGE"] = L["Mages"], ["WARLOCK"] = L["Warlocks"], ["DRUID"] = L["Druids"]}) do
				self.options.args.sets.args["set".. i].args["frame".. id].args["classes"].args[eclass] = {}
				self.options.args.sets.args["set".. i].args["frame".. id].args["classes"].args[eclass].type = 'toggle'
				self.options.args.sets.args["set".. i].args["frame".. id].args["classes"].args[eclass].name = class
				self.options.args.sets.args["set".. i].args["frame".. id].args["classes"].args[eclass].desc = class
				self.options.args.sets.args["set".. i].args["frame".. id].args["classes"].args[eclass].get = function()
																																																			local groupFilter = { strsplit(",", data.attributes.groupFilter) }
																																																			for _, _class in pairs(groupFilter) do
																																																				if _class == eclass then
																																																					return true
																																																				end
																																																			end
																																																			return false
																																																		end
				self.options.args.sets.args["set".. i].args["frame".. id].args["classes"].args[eclass].set = function(value)
																																																			local groupFilter = { strsplit(",", data.attributes.groupFilter) }
																																																			if value then
																																																				table.insert(groupFilter, eclass)
																																																			else
																																																				classes = table_removeByValue(groupFilter, eclass)
																																																			end

																																																			data.attributes.groupFilter = table.concat(groupFilter, ",")
																																																			self:RegisterGroupFilter(name, id, data.caption, data.attributes)
																																																			self:chatUpdateFilterMenu()
																																																			self:SetGroupFilters()
																																																		 end
			end
			self.options.args.sets.args["set".. i].args["frame".. id].args["groups"] = {}
			self.options.args.sets.args["set".. i].args["frame".. id].args["groups"].type = 'group'
			self.options.args.sets.args["set".. i].args["frame".. id].args["groups"].name = L["Groups"]
			self.options.args.sets.args["set".. i].args["frame".. id].args["groups"].desc = L["Groups contained in this frame"]
			self.options.args.sets.args["set".. i].args["frame".. id].args["groups"].args = {}

			for group = 1, 8 do
				self.options.args.sets.args["set".. i].args["frame".. id].args["groups"].args["group".. group] = {}
				self.options.args.sets.args["set".. i].args["frame".. id].args["groups"].args["group".. group].type = 'toggle'
				self.options.args.sets.args["set".. i].args["frame".. id].args["groups"].args["group".. group].name = string.format(L["Group %d"],group)
				self.options.args.sets.args["set".. i].args["frame".. id].args["groups"].args["group".. group].desc = string.format(L["Group %d"],group)
				self.options.args.sets.args["set".. i].args["frame".. id].args["groups"].args["group".. group].get = function()
																																																			local groupFilter = { strsplit(",", data.attributes.groupFilter) }
																																																			for _, _group in pairs(groupFilter) do
																																																				if tonumber(_group) == group then
																																																					return true
																																																				end
																																																			end
																																																			return false
																																																		end
				self.options.args.sets.args["set".. i].args["frame".. id].args["groups"].args["group".. group].set = function(value)
																																																			local groupFilter = { strsplit(",", data.attributes.groupFilter) }
																																																			if value then
																																																				table.insert(groupFilter, tostring(group))
																																																			else
																																																				groupFilter = table_removeByValue(groupFilter, tostring(group))
																																																			end

																																																			data.attributes.groupFilter = table.concat(groupFilter, ",")
																																																			self:RegisterGroupFilter(name, id, data.caption, data.attributes)
																																																			self:chatUpdateFilterMenu()
																																																			self:SetGroupFilters()
																																																		 end
			end
		end
		i = i + 1
	end
end

function table_removeByValue(tbl, value)
	for k,v in pairs(tbl) do
		if v == value then
			table.remove(tbl, k)
		end
	end
	return tbl
end